import pandas as pd
import skimage.io
from pathlib import Path
import sys

if __name__ == "__main__":
    _, shapes_file, training_images_folder = sys.argv
    
    # All the crops should be in the training image folder already
    image_files = list(Path(training_images_folder).iterdir())
    df = pd.read_csv(shapes_file).drop(columns=['shape-type', 'vertex-index'])
    image_shapes = [skimage.io.imread(image_file).shape for image_file in image_files]

    print(len(image_files))
    print(len(image_shapes))

    for axis_zero, (image_file, [rx, ry]) in enumerate(zip(image_files, image_shapes)):
        label_out_path = image_file.parents[1] / 'labels' / f"{image_file.stem}.txt"

        sub_df = df[df['axis-0'] == axis_zero]
        if len(sub_df) == 0:
            pd.DataFrame({}).to_csv(label_out_path, header=None, index=None, sep=' ', mode='w')
        else:
            with open(label_out_path, 'w') as f:
                polygon_indeces = sub_df['index'].unique().tolist()
                for polygon_idx in polygon_indeces:
                    polygon_df = df[df['index'] == polygon_idx]
                    polygon_coords_x_normed = polygon_df['axis-1'].to_numpy() / rx
                    polygon_coords_y_normed = polygon_df['axis-2'].to_numpy() / ry
                    line = "0" + f"".join(f" {y:.3f} {x:.3f}" for x, y in zip(polygon_coords_x_normed, polygon_coords_y_normed)) + "\n"
                    f.write(line)
                
                print("Saved ", label_out_path)


from tau_fibrils_yolo._widget import YoloDetectorWidget
from tau_fibrils_yolo.predict import FibrilsDetector

from ._version import version as __version__
try:
    from ._version import version as __version__
except ImportError:
    __version__ = "unknown"


import sys
import shutil
import numpy as np
from pathlib import Path

if __name__ == '__main__':
    _, data_folder, valid_fraction = sys.argv
    valid_fraction = float(valid_fraction)

    train_folder = Path(data_folder) / 'train'
    valid_folder = Path(data_folder) / 'valid'

    all_files = list((train_folder / 'labels').glob('*.txt'))

    n_files = len(all_files)
    print(f"{n_files=}")

    n_val = int(n_files * valid_fraction)
    print(f"{n_val=}")
    
    shuffled_idx = np.arange(n_files)
    np.random.shuffle(shuffled_idx)
    valid_idx = shuffled_idx[:n_val]
    train_idx = shuffled_idx[n_val:]

    for idx, file in enumerate(all_files):
        if idx in valid_idx:
            image_file = train_folder / 'images' / f"{file.stem}.tif"
            shutil.move(file, valid_folder / 'labels')
            shutil.move(image_file, valid_folder / 'images')


from dask import delayed
import dask.array as da
import numpy as np
import napari
import skimage.io
from pathlib import Path
import sys

if __name__=='__main__':
    _, training_images_folder = sys.argv
    folder = Path(training_images_folder)
    files = folder.glob("*.tif")

    lazy_imread = delayed(skimage.io.imread)
    lazy_arrays = [lazy_imread(fn) for fn in files]
    dask_arrays = [
        da.from_delayed(
            delayed_reader, 
            shape=(640, 640),
            dtype=np.uint16,
        )
        for delayed_reader in lazy_arrays
    ]
    stack = da.stack(dask_arrays, axis=0)

    viewer = napari.view_image(stack, multiscale=False, contrast_limits=[0, 65_000], colormap='gray')
    
    napari.run()


import napari
from tau_fibrils_yolo import YoloDetectorWidget, __version__

if __name__ == "__main__":
    viewer = napari.Viewer(title=f"Tau Fibrils Yolo ({__version__})")
    viewer.window.add_dock_widget(YoloDetectorWidget(viewer), name="Tau Fibrils Yolo")
    napari.run()
